% merrytest.m
% (used to be merry3.m)
% implements the MERRY algorithm for an ADSL system
% The code loops through different SNR values,
% plots performance vs. SNR,
% and plots performance vs. time for the last SNR value
%
% Written by Rick Martin, 02/06/2002
% Last updated 08/08/2002
% Cornell University Blind Equalization Research Group
%
% Copyright 2002 Cornell University, All Rights Reserved
%
% SOFTWARE shall mean "merrytest.m", Cornell Research Foundation
% ("CRF") Docket D-3010.  SOFTWARE is made available to allow
% non-commercial and research use of SOFTWARE.  Cornell
% University specifically reserves all commercial rights to
% SOFTWARE and these rights may be licensed by CRF on behalf
% of Cornell University to third parties.
%
% CRF provides SOFTWARE on an ``as is'' basis.  CRF does not
% warrant, guarantee, or make any representations regarding
% the use or results of SOFTWARE with respect to its
% correctness, accuracy, reliability or performance.  The
% entire risk of the use and performance of SOFTWARE is
% assumed by EVALUATOR.  ALL WARRANTIES INCLUDING, WITHOUT
% LIMITATION, ANY WARRANTY OF FITNESS  FOR A PARTICULAR
% PURPOSE OR MERCHANTABILITY ARE HEREBY EXCLUDED.

% Miscellaneous functions
addpath ../mcmtools

% system parameters
N  = 512;              % FFT size
nu = 32;               % CP length
nuh = nu-1;            % (ignore this until future papers)
S  = N + nu;           % total symbol length
T  = 16;               % equalizer length
% channel from DMT-TEQ toolbox (provided by ESPL at U.T. Austin)
load channels/csaloop1.time
h = (csaloop1(:,2));
h = h/norm(h);
L = length(h);         % channel length
sx = 1;                % sigma_x
ssx = sx*sx;           % signal power

% choose one of the following lines and comment out the other.
% the first line uses only one SNR value
% the second line uses many and plots performance vs. SNR
SNRs = 40;             % in dB
%SNRs = [20:1/2:50];    % in dB

% simulation parameters
% note that gearshifting is used.  mu drops by 1/2 every 1000 symbols.
%iter = 1000;           % number of symbols
iter = 5000;           % number of symbols
mu0 = 0.75;            % TEQ adaptation stepsize
starttime = 2;         % index of first usable symbol

% Optimal Melsa (MSSNR) solution
% Taken from DMT-TEQ toolbox, provided by the Embedded
%   Signal Processing Laboratory, University of Texas at Austin
% Variable names ending in "tb" indicate that they correspond
%   to this solution from the toolbox
[wtb,dopt,Dv] = mssnr(h,nu,T,1,150);
ctb = conv(wtb,h);
[SSNRtb,dummy] = Jssnr(ctb,nu,0);

% set symbol synchronization parameter
% (assume it is known)
[dummy, index] = max(abs(h));
index = index-1;       % (channel delay)
% Aim for Melsa's solution:
delta = dopt;

% equalizer delay (for initialization)
%[dummy,deq] = max(abs(wtb));
%deq = deq-1;
deq = 7;   % selected abritrarily because it works...

% equalizer initialization
w = zeros(T,iter);
winit = zeros(T,1);
winit(1+deq) = 1;
%winit = wtb;  % to start at the MSSNR solution

% allocate memory
y = zeros(1,S*iter);
c = zeros(T+L-1,iter);
SSNR = zeros(1,iter);
SSNR2 = zeros(1,iter);
wind = zeros(1,iter);
Jm = zeros(1,iter);
b = zeros(1,iter);
Jmerry_final = zeros(1,length(SNRs));
Jmelsa_final = zeros(1,length(SNRs));
BRmerry_final = zeros(1,length(SNRs));
BRmelsa_final = zeros(1,length(SNRs));

for SNRind = 1:length(SNRs)  
%--------------------------------------------------
SNR = SNRs(SNRind)
g = 10.^(-SNR/10);     % 1/SNR in linear scale
sn = sx*sqrt(g);       % sigma_n  
ssn = sn*sn;           % noise power
Rn = ssn*eye(T);       % noise autocorrelation matrix

% Performance of Melsa's solution for this SNR
Jmtb = Jmerry(ctb,wtb,nu,dopt,ssx,Rn);
btb = sum( bitrate(ctb,wtb,nu,dopt,ssx,ssn) );

% Optimal Merry solution
[wopt,SSNRopt] = merryopt(h,nu-1,T,dopt+1,g);
copt = conv(wopt,h);
[SSNRopt,dummy] = Jssnr(copt,nu,0);
copt = conv(wopt,h);
Jmopt = Jmerry(copt,wopt,nu,dopt,ssx,Rn);
bopt = sum( bitrate(copt,wopt,nu,dopt,ssx,ssn) );

% Generate ADSL received data
r = adslout(h,sx,sn,iter);

% initialize equalizer
w(:,starttime) = winit;

% reset stepsize to initial value
% (it may be reduced for gearshifting later)
mu = mu0;

% run algorithm
hwin = waitbar(0,'Running MERRY...');
for k=starttime:iter-1

   % gearshifting
   % (high stepsize initially for fast convergence,
   % low stepsize in the end for low misadjustment)
   % (this will not affect the first 1000 iterations)
   if mod(k,1000) == 0, mu = mu/2; end

   c(:,k) = (conv( w(:,k), h));
   SSNR(k) = Jssnr(c(:,k),nu,0,delta+1,delta+1);
   [SSNR2(k),wind(k)] = Jssnr(c(:,k),nu,0);
   Jm(k) = Jmerry(c(:,k),w(:,k),nu,dopt,ssx,Rn);
   b(k) = sum( bitrate(c(:,k),w(:,k),nu,dopt,ssx,ssn) );

   % convolve w with r to get y (for current and next symbols)
   y(S*(k-1)+[1:S]) = zeros(1,S);
   y(S*(k-1)+[1:S]+S) = zeros(1,S);
   for j=0:T-1
      y(S*(k-1)+[1:S]) = y(S*(k-1)+[1:S]) + ...
                           w(j+1,k) * r(S*(k-1)+[1:S]-j);
      y(S*(k-1)+[1:S]+S) = y(S*(k-1)+[1:S]+S) + ...
                           w(j+1,k) * r(S*(k-1)+[1:S]-j+S);
   end

   % update w
   for j=0:T-1
      dJdw = 0;
      for i = S*(k-1)+1+nuh+delta : S*(k-1)+nu+delta
         dJdw = dJdw + (y(i)-y(i+N)) * conj((r(i-j)-r(i+N-j)));
      end
      w(j+1,k+1) = w(j+1,k) - mu * dJdw;
   end

   % normalize
   w(:,k+1) = w(:,k+1)/norm(w(:,k+1));
   %w(:,k+1) = w(:,k+1)/norm( conv(w(:,k+1),h) );

   waitbar(k/iter,hwin);
end
close(hwin);
c(:,k+1) = conv( w(:,k+1), h);
SSNR(k+1) = Jssnr(c(:,k),nu,0,delta+1,delta+1);
[SSNR2(k+1),wind(k+1)] = Jssnr(c(:,k),nu,0);
Jm(k+1) = Jmerry(c(:,k+1),w(:,k+1),nu,dopt,ssx,Rn);
b(k+1) = sum( bitrate(c(:,k+1),w(:,k+1),nu,dopt,ssx,ssn) );

% final performance for this SNR value
Jmerry_final(SNRind) = Jm(k+1);
Jmelsa_final(SNRind) = Jmtb;
BRmerry_final(SNRind) = b(k+1);
BRmelsa_final(SNRind) = btb;
%--------------------------------------------------
end

% The code is set up to plot learning curves and impulse
% response plots for the final SNR value, as well as
% the performance vs. SNR curves.  This code is clearly
% not optimized; feel free to recode it.

% transparent mask to overlay on impulse response plots
Background = .8*ones(1,3);

% impulse response plots
figure(1);
clf
set(0,'DefaultAxesFontSize',14);
%
subplot(4,1,1);
plot([0:T+L-2],c(:,starttime)/max(abs(c(:,starttime))),'r-');
title('Channel-Equalizer Combinations','FontSize',16);
ylabel('initial','FontSize',16);
v = axis;
xdata = [delta delta+nu-1 delta+nu-1 delta];
ydata = [v(3) v(3) v(4) v(4)];
patch('Xdata',xdata,'Ydata',ydata,...
      'EraseMode','xor','EdgeColor','w','FaceColor',Background);
axis([0 200 v(3) v(4)]);
%
subplot(4,1,2);
plot([0:T+L-2],c(:,end)/max(abs(c(:,end))),'r-');
ylabel('final','FontSize',16);
v = axis;
xdata = [delta delta+nu-1 delta+nu-1 delta];
ydata = [v(3) v(3) v(4) v(4)];
patch('Xdata',xdata,'Ydata',ydata,...
      'EraseMode','xor','EdgeColor','w','FaceColor',Background);
axis([0 200 v(3) v(4)]);
%
subplot(4,1,3);
plot([0:T+L-2],copt/max(abs(copt)),'r-');
ylabel('Merry Opt.','FontSize',16);
v = axis;
xdata = [delta delta+nu-1 delta+nu-1 delta];
ydata = [v(3) v(3) v(4) v(4)];
patch('Xdata',xdata,'Ydata',ydata,...
      'EraseMode','xor','EdgeColor','w','FaceColor',Background);
axis([0 200 v(3) v(4)]);

subplot(4,1,4);
plot([0:T+L-2],ctb/max(abs(ctb)),'r-');
ylabel('Melsa','FontSize',16);
v = axis;
xdata = [delta delta+nu-1 delta+nu-1 delta];
ydata = [v(3) v(3) v(4) v(4)];
patch('Xdata',xdata,'Ydata',ydata,...
      'EraseMode','xor','EdgeColor','w','FaceColor',Background);
axis([0 200 v(3) v(4)]);
xlabel('tap index','FontSize',14);

% plots of 1 / Shortening SNR vs. time
figure(2);
clf
set(0,'DefaultAxesFontSize',14);
plot([starttime:iter],10*log10(SSNR2(starttime:iter)),'b-',...
     [starttime,iter],10*log10(SSNRtb)*[1,1],'k:',...
     [starttime,iter],10*log10(SSNRopt)*[1,1],'k--');
ylabel('1 / Shortening SNR of system, in dB','FontSize',14);
xlabel('symbol index','FontSize',14);
legend('Adapted','Melsa','Optimal MERRY',0);

% plots of achievable bit rate vs. time
Tsym = 246.4e-6;
figure(3);
clf
set(0,'DefaultAxesFontSize',9);
subplot(2,1,1);
semilogy([starttime:iter],Jm(starttime:iter),'b-',...
     [starttime,iter],Jmopt*[1,1],'k--','LineWidth',2);
ylabel('MERRY cost','FontSize',14);
%legend('Adapted','Optimal MERRY',1);
legend('ADAPTED','OPTIMAL MERRY',1);
grid on
%axis([0 iter 5e-5 2e-1]);
axis([0 iter 0.85e-4 1]);
%
subplot(2,1,2);
plot([starttime:iter],b(starttime:iter)/Tsym,'b-',...
     [starttime,iter],bopt/Tsym*[1,1],'k--',...
     [starttime,iter],btb/Tsym*[1,1],'k:','LineWidth',2);
ylabel('bit rate (bps)','FontSize',14);
xlabel('symbol index','FontSize',14);
%legend('Adapted','Optimal MERRY','Max SSNR',4);
legend('ADAPTED','OPTIMAL MERRY','MAX SSNR',4);
grid on

% more impulse response plots
figure(4);
clf
set(0,'DefaultAxesFontSize',14);
subplot(2,1,1);
plot([0:T+L-2],c(:,starttime)/max(abs(c(:,starttime))),'r-');
title('Channel-Equalizer Combinations','FontSize',16);
ylabel('initial','FontSize',16);
v = axis; 
v = [v(1) v(2) -0.2 1];
axis([0 200 v(3) v(4)]);
hold on;
plot([delta delta],[v(3) v(4)],'k--',...
   [delta+nu-1 delta+nu-1],[v(3) v(4)],'k--',...
   [0 200],[0 0],'k-');
hold off;
%
subplot(2,1,2);
plot([0:T+L-2],c(:,end)/max(abs(c(:,end))),'r-');
ylabel('final','FontSize',16);
v = axis; 
axis([0 200 v(3) v(4)]);
hold on; plot([delta delta],[v(3) v(4)],'k--',...
   [delta+nu-1 delta+nu-1],[v(3) v(4)],'k--',...
   [0 200],[0 0],'k-');
hold off;
xlabel('tap index','FontSize',16);

% performance vs. SNR
figure(5);
clf
set(0,'DefaultAxesFontSize',14);
plot(SNRs,BRmerry_final/Tsym,'b-',...
     SNRs,BRmelsa_final/Tsym,'k--','LineWidth',2);
v = axis; 
axis([v(1) v(2) 0 v(4)]);
xlabel('SNR, in dB','FontSize',14);
ylabel('bit rate (bps)','FontSize',14);
title('Performance after 5000 iterations','FontSize',14);
legend('MERRY','Max SSNR',2);
grid on
